/*********************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : Dec. 2006

 File          : FlashEE.c

 Hardware      : Applicable to ADuC703x

 Description   : In this example a page is dedicated to calibration data 
				 This page is read back and send through the UART at 115200 bps 
		
*********************************************************************/
#include <ADuC7033.h>

extern int write (int file, char * ptr, int len);	// Function used to write string

void delay (int length);

void erase_page(unsigned short int addr);
void save(unsigned short int addr, unsigned char data);
unsigned short load(unsigned short int addr);

void senddata(short);
char hex2ascii(char);

unsigned char ERROR;
unsigned char status;

const char MyTable[0x200] __attribute__ ((section (".go_where_I_want_you"))) = { 0xFF };

int main(void)	 
{
	unsigned int i = 0;
	ERROR = 0;

	GP1CON = 0x011;	   			// Setup tx & rx pins on P1.0 and P1.1
  	COMCON0 = 0x80;				// Setting DLAB
	COMDIV0 = 0x02;				// 115200 bps
   	COMDIV1 = 0x00;
   	COMCON0 = 0x07;				// Clearing DLAB
  	COMDIV2 = 0x8B1C;

	GP2DAT = 0x02000000;		// Enable P2.1 (GPIO8) as an output
	FEE1MOD = 0x8;				// bit 3 should be set to allow erase/write command


	for (i=0;i<0x200;i++){			// Output Data
	   	senddata (MyTable[i]);
	} 

	while (1){
		GP2DAT ^=0x020000;		// Complement P2.1
		delay(40000);
	}
    
	return 0;
}

void delay (int length) {		// delay
	while (length >= 0)
	    length--;
}

void protect_page(unsigned int addr){
	FEE1ADR = 0x1234;			 		// Key
	FEE1DAT = 0xA5A5;					// Key
	FEE1PRO = addr;
	FEE1MOD = 0x48;
	FEE1CON = 0x0C;
	status = FEE1STA&0x03;
	while (!(status)) status = FEE1STA&0x03;
	if ((status&0x02)==0x02) ERROR = 1;
	return;								// return 
} 


unsigned short load(unsigned short int addr){
	FEE1ADR = addr;
	FEE1CON = 0x01;				// single read command
	status = FEE1STA&0x03;
	while (!(status)) status = FEE1STA&0x03;
	if ((status&0x02)==0x02) ERROR = 1;
	return (FEE1DAT);		
} 


void save(unsigned short int addr, unsigned char data){
	FEE1ADR = addr;				// set data address
	FEE1DAT = data;				// set data value
	FEE1CON = 0x02;				// single Write command
	status = FEE1STA&0x03;
	while (!(status)) status = FEE1STA&0x03;
	if ((status&0x02)==0x02) ERROR = 1;
	return;
}

void erase_page(unsigned short int addr){
	FEE1ADR = addr;				// set data address
	FEE1CON = 0x05;				// erase page command
	status = FEE1STA&0x03;
	while (!(status)) status = FEE1STA&0x03;
	if ((status&0x02)==0x02) ERROR = 1;
 	return;
}

void senddata(short to_send){
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = 0x0A;						// output LF 
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = 0x0D;						// output CR 
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = hex2ascii ((to_send >> 12) & 0x0F);
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = hex2ascii ((to_send >> 8) & 0x0F);
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = hex2ascii ((to_send >> 4) & 0x0F);						
	while(!(0x020==(COMSTA0 & 0x020))){}
		COMTX = hex2ascii (to_send & 0x0F);							
}


char hex2ascii(char toconv){
	if (toconv<0x0A) toconv += 0x30;
	else toconv += 0x37;
	return (toconv);
}


